package com.maileon.api.account;

import com.maileon.api.AbstractMaileonService;
import com.maileon.api.MaileonConfiguration;
import com.maileon.api.MaileonException;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import java.io.StringReader;
import java.io.StringWriter;

/**
 * Facade that wraps the REST service for accounts.
 *
 * @author Marcus Beckerle | XQueue GmbH | <a href="mailto:marcus.beckerle@xqueue.com">marcus.beckerle@xqueue.com</a>
 */
public class MaileonAccountService extends AbstractMaileonService {

    public static final String SERVICE = "MAILEON ACCOUNT SETTINGS";

    /**
     * Constructs a <code>MaileonMailingsService</code>.
     *
     * @param config Maileon API-Configuration.
     */
    public MaileonAccountService(MaileonConfiguration config) {
        super(config, SERVICE);
    }

    /**
     * Get list of all account placeholders.
     *
     * @return
     */
    public AccountPlaceholders getAccountPlaceholders() throws MaileonException {
        try {
            String response = get("account/placeholders").getEntityAsString();
            JAXBContext jaxbContext = JAXBContext.newInstance(AccountPlaceholders.class);
            Unmarshaller jaxbMarshaller = jaxbContext.createUnmarshaller();
            StringReader reader = new StringReader(response);
            return (AccountPlaceholders) jaxbMarshaller.unmarshal(reader);
        } catch (Exception e) {
            throw new MaileonException(e.getMessage(), e);
        }
    }


    /**
     * Sets the list of account placeholders. All current account placeholders will be overwritten.
     *
     * @param accountPlaceholders account placeholders to set
     * @return
     */
    public void setAccountPlaceholders(AccountPlaceholders accountPlaceholders) throws MaileonException {
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(AccountPlaceholders.class);
            Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
            StringWriter writer = new StringWriter();
            jaxbMarshaller.marshal(accountPlaceholders, writer);
            String xml = writer.toString();

            post("account/placeholders", xml);
        } catch (Exception e) {
            throw new MaileonException(e.getMessage(), e);
        }
    }


    /**
     * Update account placeholders. If account placeholder is not existing yet, it will be added. If account placeholder with given name is avaliable the value will be updated. Other existing account placeholders will not be touched.
     *
     * @param accountPlaceholders account placeholders to add/update
     */

    public void updateAccountPlaceholders(AccountPlaceholders accountPlaceholders) throws MaileonException {
        try {
            JAXBContext jaxbContext = JAXBContext.newInstance(AccountPlaceholders.class);
            Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
            StringWriter writer = new StringWriter();
            jaxbMarshaller.marshal(accountPlaceholders, writer);
            String xml = writer.toString();

            put("account/placeholders", xml);
        } catch (Exception e) {
            throw new MaileonException(e.getMessage(), e);
        }
    }


    /**
     * Delete account placeholder with requested {@code name}
     *
     * @param name The name of the account placeholder to delete
     */
    public void deleteAccountPlaceholder(String name) throws MaileonException {
        QueryParameters parameters = new QueryParameters("name", name);
        delete("account/placeholders", parameters, null);
    }

}
