package com.maileon.api.targetgroups;

import com.maileon.api.*;
import com.maileon.api.targetgroups.xml.TargetGroupXmlSerializer;
import com.maileon.api.targetgroups.xml.TargetGroupsXmlSerializer;
import com.maileon.api.utils.PageUtils;
import org.dom4j.Element;

/**
 * The <code>TargetGroupsService</code> client send operational requests to the Maileon <code>TargetGroupsResource</code>.
 *
 */
public class TargetGroupsService extends AbstractMaileonService {

    public static final String SERVICE = "MAILEON TARGETGROUPS";

    /**
     * Instantiates a new Maileon target group service.
     *
     * @param config the configuration of Maileon API
     */
    public TargetGroupsService(MaileonConfiguration config) {
        super(config, SERVICE);
    }


    /**
     * Returns the total count of target groups in the account. See <a href="http://dev.maileon.com/api/rest-api-1-0/targetgroups/get-targetgroups-count/?lang=en">Maileon API documentation</a>.
     *
     * @throws MaileonException
     */
    public int getTargetGroupsCount() throws MaileonException {
        ResponseWrapper response = get("targetgroups/count");
        try {
            return Integer.parseInt(response.getEntityAsXml().getText());
        } catch (Throwable t) {
            throw new MaileonException("Unable to parse target groups count as integer: " + response.getEntityAsXml().getText(), t);
        }
    }

    /**
     * Retuns a page of contact filters using the provided pagination settings. See <a href="http://dev.maileon.com/api/rest-api-1-0/targetgroups/get-targetgroups/?lang=en">Maileon API documentation</a>.
     *
     * @param pageSize required number of elements in the result page [1..1000].
     * @param pageIndex starts with 1.
     * @return a page containing the returned {@link TargetGroup} objects.
     * @throws MaileonException
     */
    public Page<TargetGroup> getTargetGroups(int pageSize, int pageIndex) throws MaileonException {
        QueryParameters qp = new QueryParameters("page_size", pageSize);
        qp.add("page_index", pageIndex);

        ResponseWrapper response = get("targetgroups", qp);

        try {
            Page<TargetGroup> page = PageUtils.createPage(pageIndex, pageSize, response);

            Element xml = response.getEntityAsXml();
            page.setItems(TargetGroupsXmlSerializer.deserialize(xml));

            return page;
        } catch (Exception e) {
            throw new MaileonException("Problem while executing GET on all target groups", e);
        }
    }

    /**
     * Gets a targetgroup according to a particular id.
     *
     * @param targetGroupId of the requested targetgroup ID
     * @return the {@link com.maileon.api.targetgroups.TargetGroup} instance
     * @throws MaileonException
     */
    public TargetGroup getTargetGroup(long targetGroupId) throws MaileonException {
        ResponseWrapper response = get("targetgroups/targetgroup/" + targetGroupId);

        try {
            return TargetGroupXmlSerializer.deserializeSingleElement(response.getEntityAsXml());
        } catch (Exception e) {
            throw new MaileonException("Problem while executing GET on single target group with ID " + targetGroupId, e);
        }
    }

    /**
     * Deletes a targetgroup according to a particular id.
     *
     * @param targetGroupId ID of the targetgroup to delete
     * @return the {@link com.maileon.api.targetgroups.TargetGroup} instance that has been deleted
     * @throws MaileonException
     */
    public TargetGroup deleteTargetGroup(long targetGroupId) throws MaileonException {
        ResponseWrapper response = delete("targetgroups/targetgroup/" + targetGroupId);

        try {
            return TargetGroupXmlSerializer.deserializeSingleElement(response.getEntityAsXml());
        } catch (Exception e) {
            throw new MaileonException("Problem while executing GET on single target group with ID " + targetGroupId, e);
        }
    }
}
